# Copyright © 2020 Hashmap, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from servz.core.build_step import BuildStep
from servz.utils.exceptions.flow_build_error import FlowBuildError


class PipelineComposer(BuildStep):

    def __init__(self, **kwargs):
        super().__init__()
        self._pipelines: list = []

    def build_part(self, **kwargs):
        _res = self._build(flows=kwargs.get("workflows"))

        if not _res:
            raise FlowBuildError()

    def get_results(self):

        result = dict()
        if self._pipelines:
            result = {
                "pipelines": self._pipelines
            }

        return result

    def _build(self, flows: dict) -> bool:
        raise NotImplementedError(f'Method not implemented for {type(self).__name__}.')
