# Modifications © 2020 Hashmap, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import logging


class MLError(Exception):
    def __init__(self, *args):
        if args:
            self.message = args[0]
        else:
            self.message = None

        self._logger = logging.getLogger(__name__)
        self._logger.setLevel(logging.INFO)

    def __str__(self):
        if self.message:
            msg = f'{type(self).__name__}, {self.message}'
        else:
            msg = f'{type(self).__name__} has been raised.'
        self._logger.exception(msg)
        return msg
