# Copyright © 2020 Hashmap, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from servz.core.package_publisher.package_publisher import PackagePublisher


class ZipfilePublisher(PackagePublisher):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def build_part(self, **kwargs):

        result = self.__artifact_registry.register('test', artifact={
            'filename': kwargs['zip_file']
        })

        self._build_flows.append({'register_result': result})

    def get_results(self):

        return {
            "package_publisher": self._build_flows
        }
